#ifndef TCURSES_H
#define TCURSES_H

typedef struct tcurses_cell_s {
	char fg, bg, attr;
	char c;              /* the actual character printed on the cell */
} tcurses_cell_t;

typedef struct tcurses_term_s {
	int fdo;
	int sx, sy;
	tcurses_cell_t *current, *last;
	int x, y;     /* logical cursor location */
	int pcx, pcy; /* physical cursor location */
	int fg, bg, attr; /* drawing attrs */
	int state;
	void *driver;
} tcurses_term_t;

extern const char **tcurses_cap_search_user; /* caller may fill this in with a NULL terminated array of search paths for project-local tcurses caps dirs */
extern const char *tcurses_cap_search_sys[]; /* read-only: system tcap search paths; searched if the terminal is not found in user search path */

/* when sx and sy are non-zero, no autodetection is performed */
tcurses_term_t *tcurses_init(int fdo, const char *term_type, int sx, int sy);

void tcurses_uninit(tcurses_term_t *term);

/* return 1 if initialization has finished */
int tcurses_inited(tcurses_term_t *term);

/* The caller reads the input stream and passes on characters to tcurses.
   The caller also calls tcurses_getch with TCURSES_TICK from time to
   time when no character is available (for escape delay multichar support).
   The caller needs to call tcurses_getch() with TCURSES_EOF on eof.
   The call may return TCURSES_NOTHING when no character is availble yet */
#define TCURSES_TICK
#define TCURSES_EOF
#define TCURSES_NOTHING
int tcurses_getch(tcurses_term_t *term, int ch);

/* calculate and write all changes to the screen */
void tcurses_flush(tcurses_term_t *term);

/* move the physicial cursor after flush (the blinking thing) - does not
   affect the logical cursor */
void tcurses_cursor(tcurses_term_t *term, int col, int row);

/* clear the entire screen */
void tcurses_clear(tcurses_term_t *term);

/* move the logical cursor (for internal screen manipulation) */
void tcurses_gotoxy(tcurses_term_t *term, int col, int row);

/* low level attr set */
void tcurses_setattr(tcurses_term_t *term, char fg, char bg, char a);

/* print a string at current logical cursor position with current attrs */
void tcurses_write(tcurses_term_t *term, char *str);
void tcurses_write_len(tcurses_term_t *term, char *str, int len);

/* Chnage terminal size and clear */
void tcurses_resize(tcurses_term_t *term, int sx, int sy);

/* debug */
void tcurses_dump_to_file(tcurses_term_t *term, const char *fn);

#endif
